function [bestBox, E] = ml_findBox1b(kpAssign, imH, imW, binWeights, szRange)
% function [bestBox, E] = ml_findBox1b(kpAssign, imH, imW, binWeights)
% function [bestBox, E] = ml_findBox1b(kpAssign, imH, imW, binWeights, szRange)
% Find a rectangle with the histogram that minimize the weighted sum
%   sum_i {binWeights(i)*hist(i)}
% Inputs:
%   kpAssign: a matrix for histogram bin assignments of keypoints. 
%       kpAssign(i,1), kpAssign(i,2), kpAssign(i,3) are the row#, col#,
%       bin#. 1 <= row# <= imH; 1 <= col# <= imW; 1 <= bin# <= k, where k
%       is the length of binWeights.
%   imH, imW: height and width of the image
%   binWeights: a k*1 vector of weights for histogram bins, weights can be
%       negative
%	szRange: a 4*1 vector for [wmin; wmax; hmin; hmax], range for width and height
% Output:
%   bestBox: a 4*1 vector for the left, top, right, bottom of the rectangle
%   E: energy
% Examples:
%   imH = 100; imW = 200;
%   imAssign = ones(imH, imW);
%   imAssign(50:70,100:150) = 2;
%   [IX,IY] = meshgrid(1:imW, 1:imH);
%   kpAssign = [IY(:), IX(:), imAssign(:)];
%   [bbox, E] = ml_findBox1b(kpAssign, imH, imW, [100;-1]);
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Date: 16 Nov 2008
% Last modified: 19 Feb 09

if size(binWeights, 2) ~= 1
    error('binWeights should be a column vector');
end;

nHistBin = size(binWeights, 1);
if sum(kpAssign(:) <= 0) > 0
    error('entries of kpAssign should be positive integers');
end

if sum(kpAssign(:,1) > imH) > 0
    error('entries of kpAssign(:,1) must be <= imH');
end;

if sum(kpAssign(:,2) > imW) > 0
    error('entries of kpAssign(:,2) must be <= imW');
end;
if sum(kpAssign(:,3) > nHistBin) > 0
    error('entries of kpAssign(:,3) must be smaller than the length of binWeights');
end

if exist('szRange','var') && ~isempty(szRange)
    if (size(szRange, 1) ~= 4) || (size(szRange, 2) ~= 1)
        error('szRange should be a 4*1 vector');
    end;

    if (szRange(1) > szRange(2)) || (szRange(3) > szRange(4)) ||...
        szRange(2) <= 0 || szRange(4) <=0 || ...
        szRange(1) > imW || szRange(3) > imH
        error('invalid size range');
    end;
    [bestBox, E] = m_mexFindBox1b(kpAssign, imH, imW, binWeights, szRange);
else
    [bestBox, E] = m_mexFindBox1b(kpAssign, imH, imW, binWeights);
end;
    




